<?php /** @noinspection SpellCheckingInspection */

namespace King_Addons;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

final class ModulesMap
{
    public static function getModulesMapArray(): array
    {
        return [
            'widgets' => [
                'styled-text-builder' => [
                    'title' => esc_html__('Styled Text Builder', 'king-addons'),
                    'description' => esc_html__('Enables the creation of uniquely styled and animated text elements, offering extensive customization options for impactful and visually captivating web typography. Allows combining different styles in one text paragraph.', 'king-addons'),
                    'php-class' => 'Styled_Text_Builder',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/styled-text-builder/',
                    'css' => ['style'],
                    'js' => [],
                ],
                'image-hotspots' => [
                    'title' => esc_html__('Image Hotspots', 'king-addons'),
                    'description' => esc_html__('Module for Elementor to create interactive, clickable areas on images, ideal for displaying information or links in a dynamic, visually engaging way. Uses only CSS, not any JS.', 'king-addons'),
                    'php-class' => 'Image_Hotspots',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/image-hotspots/',
                    'css' => ['style'],
                    'js' => [],
                ],
                'one-page-navigation' => [
                    'title' => esc_html__('One Page Navigation', 'king-addons'),
                    'description' => esc_html__('Facilitates seamless navigation within a single page, using anchor links to smoothly scroll to different sections, enhancing user experience and site organization. Uses only CSS, not any JS.', 'king-addons'),
                    'php-class' => 'One_Page_Navigation',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/one-page-navigation/',
                    'css' => ['style'],
                    'js' => [],
                ],
                'off-canvas-content' => [
                    'title' => esc_html__('Off-Canvas Content ', 'king-addons'),
                    'description' => esc_html__('Allows the creation of hidden, slide-in panels on websites, ideal for menus, widgets, or additional content, accessible with a simple user interaction.', 'king-addons'),
                    'php-class' => 'Off_Canvas_Content',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/off-canvas-content/',
                    'css' => ['style'],
                    'js' => [],
                ],
                'image-comparison' => [
                    'title' => esc_html__('Image Comparison', 'king-addons'),
                    'description' => esc_html__('Also known as Before & After, the widget offers an interactive tool to visually contrast two images side-by-side with a draggable slider, perfect for before-and-after views, product comparisons, or design showcases.', 'king-addons'),
                    'php-class' => 'Image_Comparison',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/image-comparison/',
                    'css' => ['style'],
                    'js' => [],
                ],
                'vertical-scrollable-image' => [
                    'title' => esc_html__('Vertical Scrollable Image', 'king-addons'),
                    'description' => esc_html__('Widget allows seamless vertical scrolling on mouse hover over through long images, such as infographics or timelines, on a webpage, enhancing user engagement and storytelling without resizing. Uses only CSS, not any JS.', 'king-addons'),
                    'php-class' => 'Vertical_Scrollable_Image',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/vertical-scrollable-image/',
                    'css' => ['style'],
                    'js' => [],
                ],
                'global-section-container' => [
                    'title' => esc_html__('Global Section & Container', 'king-addons'),
                    'description' => esc_html__('Allows designers to create and manage reusable website sections and containers, streamlining design consistency and efficiency across multiple pages. Useful for building Header, Footer, and CTA sections. Does not affect website performance because it uses Elementor built-in functionality. Does not use any CSS or JS.', 'king-addons'),
                    'php-class' => 'Global_Section_Container',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/global-section-container/',
                    'css' => [],
                    'js' => [],
                ],
                'card-carousel' => [
                    'title' => esc_html__('Card Carousel', 'king-addons'),
                    'description' => esc_html__('Create a card carousel, also known as horizontal slider/scroller. Each card has an image, title, subtitle, description and button. Ideal for showcasing content, highlighting features, or presenting services, perfect for team members and testimonials sections. It is fully responsive, and supports both mobile and mouse dragging.', 'king-addons'),
                    'php-class' => 'Card_Carousel',
                    'docs-link' => '',
//                    'demo-link' => 'https://kingaddons.com/elementor/card-carousel/',
                    'demo-link' => '',
                    'css' => ['style'],
                    'js' => [],
                ],
                'auto-scrolling-text' => [
                    'title' => esc_html__('Auto-Scrolling Text', 'king-addons'),
                    'description' => esc_html__('An engaging text marquee for dynamic, continuously scrolling text sections. Ideal for announcements, news tickers, or any content you want in constant motion. Customize the style with various effects, and add images to enhance your design. Uses only CSS for animation.', 'king-addons'),
                    'php-class' => 'Auto_Scrolling_Text',
                    'docs-link' => '',
//                    'demo-link' => 'https://kingaddons.com/elementor/auto-scrolling-text/',
                    'demo-link' => '',
                    'css' => ['style'],
                    'js' => [],
                ],
                'popup' => [
                    'title' => esc_html__('Popup & Lightbox Modal', 'king-addons'),
                    'description' => esc_html__('A versatile tool for creating engaging popups and lightbox modals. Perfect for displaying important messages, promotions, or multimedia content. Customize the appearance and behavior with various effects, and easily add images, videos, and other elements to capture your audience attention.', 'king-addons'),
                    'php-class' => 'Popup',
                    'docs-link' => '',
//                    'demo-link' => 'https://kingaddons.com/elementor/auto-scrolling-text/',
                    'demo-link' => '',
                    'css' => ['style'],
                    'js' => [],
                ],
                'video-popup' => [
                    'title' => esc_html__('Video Popup', 'king-addons'),
                    'description' => esc_html__('A powerful tool for creating engaging video popups. Ideal for showcasing promotional videos, tutorials, or any multimedia content. Customize the appearance and behavior with various effects, and easily integrate videos to capture your audience attention.', 'king-addons'),
                    'php-class' => 'Video_Popup',
                    'docs-link' => '',
//                    'demo-link' => 'https://kingaddons.com/elementor/auto-scrolling-text/',
                    'demo-link' => '',
                    'css' => ['style'],
                    'js' => [],
                ],
//                'test' => [
//                    'title' => esc_html__('test', 'king-addons'),
//                    'description' => esc_html__('', 'king-addons'),
//                    'php-class' => 'Auto_Scrolling_Text',
//                    'docs-link' => '',
////                    'demo-link' => 'https://kingaddons.com/elementor/test/',
//                    'demo-link' => '',
//                    'css' => ['style'],
//                    'js' => [],
//                ],
            ],
            'features' => [
                'particles-background' => [
                    'title' => esc_html__('Particles Background', 'king-addons'),
                    'description' => esc_html__('Brings website to life by adding a dynamic, animated particle effect as background, creating an engaging and modern visual experience.', 'king-addons'),
                    'php-class' => 'Particles_Background',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/particles-background/',
                    'css' => [],
                    'js' => ['preview-handler']
                ],
                'parallax-background' => [
                    'title' => esc_html__('Parallax Background', 'king-addons'),
                    'description' => esc_html__('Adds a dynamic, multi-layered visual effect to website backgrounds, moving at different speeds during scrolling for an immersive, 3D-like experience for sections and containers. The feature uses only lightweight JS code (11 kilobytes).', 'king-addons'),
                    'php-class' => 'Parallax_Background',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/parallax-background/',
                    'css' => [],
                    'js' => ['preview-handler']
                ],
                'matte-glass-background' => [
                    'title' => esc_html__('Matte Glass Background', 'king-addons'),
                    'description' => esc_html__('Adds matte glass background feature also known as frosted glass effect. The feature uses only CSS, not any JavaScript at all.', 'king-addons'),
                    'php-class' => 'Matte_Glass_Background',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/matte-glass-background/',
                    'css' => [],
                    'js' => []
                ],
                'floating-animation' => [
                    'title' => esc_html__('Floating Animation', 'king-addons'),
                    'description' => esc_html__('Adds floating animation setting. The feature does not use JavaScript, only lightweight CSS animation.', 'king-addons'),
                    'php-class' => 'Floating_Animation',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/floating-animation/',
                    'css' => [],
                    'js' => ['preview-handler']
                ],
                'custom-css' => [
                    'title' => esc_html__('Custom CSS', 'king-addons'),
                    'description' => esc_html__('Provides an easy-to-use interface for adding personalized CSS styles, allowing precise control over the appearance and layout of web elements on your page.', 'king-addons'),
                    'php-class' => 'Custom_CSS',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/custom-css/',
                    'css' => [],
                    'js' => ['preview-handler']
                ],
                'rotating-animation' => [
                    'title' => esc_html__('Rotating Animation', 'king-addons'),
                    'description' => esc_html__('Adds rotating animation setting. The feature does not use JS, only lightweight CSS animation.', 'king-addons'),
                    'php-class' => 'Rotating_Animation',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/rotating-animation/',
                    'css' => [],
                    'js' => ['preview-handler']
                ],
                'wrapper-link' => [
                    'title' => esc_html__('Wrapper Link', 'king-addons'),
                    'description' => esc_html__('Wrap everything in link - section, container, column and common elements', 'king-addons'),
                    'php-class' => 'Wrapper_Link',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/wrapper-link/',
                    'css' => [],
                    'js' => []
                ],
                'duplicator' => [
                    'title' => esc_html__('Duplicator', 'king-addons'),
                    'description' => esc_html__('Adds feature to duplicate posts, pages, Elementor templates and etc.', 'king-addons'),
                    'php-class' => 'Duplicator',
                    'docs-link' => '',
                    'demo-link' => 'https://kingaddons.com/elementor/duplicator/',
                    'css' => [],
                    'js' => []
                ],
            ],
            'libraries' => [
                'swiper' => [
                    'css' => ['swiper'],
                    'js' => ['swiper']
                ],
                'jarallax' => [
                    'css' => [],
                    'js' => ['jarallax']
                ],
                'particles' => [
                    'css' => [],
                    'js' => ['particles']
                ],
            ]
        ];
    }
}